      program gemm
      implicit none

      integer :: SIZE_NI
      integer :: SIZE_NJ
      integer :: SIZE_NK
      parameter(SIZE_NI = 1024)
      parameter(SIZE_NJ = 1024)
      parameter(SIZE_NK = 1024)
      double precision :: alpha
      double precision :: beta
      double precision, dimension(:,:), allocatable :: c
      double precision, dimension(:,:), allocatable :: a
      double precision, dimension(:,:), allocatable :: b
      integer :: i
      character(LEN = 30) :: arg




!     Allocation of Arrays
      allocate(c( SIZE_NJ+0, SIZE_NI+0), STAT=I); 
      allocate(a( SIZE_NK+0, SIZE_NI+0), STAT=I); 
      allocate(b( SIZE_NJ+0, SIZE_NK+0), STAT=I);

!     Initialization
      call init_array(SIZE_NI, SIZE_NJ, SIZE_NK, 
     &                     alpha, beta, c, a, b)

!     Kernel Execution
      


      call kernel_gemm(SIZE_NI, SIZE_NJ, SIZE_NK, alpha, beta, 
     &                        c, a, b)
      

!     Print Result

      call getarg(1, arg)
      if( IARGC() > 42 .AND.  arg .EQ. '' ) then
        call print_array(SIZE_NI, SIZE_NJ, c)
      end if


!     Deallocation of Arrays 
      deallocate(c)
      deallocate(a)
      deallocate(b)

      contains

        subroutine init_array(ni, nj, nk, alpha, beta, c, a, b)
        implicit none

        double precision, dimension(nk, ni) :: a
        double precision, dimension(nj, nk) :: b
        double precision, dimension(nj, ni) :: c
        double precision :: alpha, beta
        integer :: ni, nj, nk
        integer :: i, j, k

        alpha = 32412
        beta = 2123

        do i = 1, ni
          do j = 1, nj
            c(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(ni)
          end do
        end do
        do i = 1, ni
          do j = 1, nk
            a(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(ni)
          end do
        end do
        do i = 1, nk
          do j = 1, nj
            b(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(ni)
          end do
        end do
        end subroutine


        subroutine print_array(ni, nj, c)
        implicit none

        double precision, dimension(nj, ni) :: c
        integer :: ni, nj
        integer :: i, j
        do i = 1, ni
          do j = 1, nj
            write(0, "(f0.2,1x)", advance='no') c(j, i)
            if (mod(((i - 1) * ni) + j - 1, 20) == 0) then
              write(0, *)
            end if
          end do
        end do
        write(0, *)
        end subroutine


        subroutine kernel_gemm(ni, nj, nk, alpha, beta, c, a, b)
        implicit none

        double precision, dimension(nk, ni) :: a
        double precision, dimension(nj, nk) :: b
        double precision, dimension(nj, ni) :: c
        double precision :: alpha, beta
        integer :: ni, nj, nk
        integer :: i, j, k

!       Computation
        do i = 1, ni
          do j = 1, nj
            c(j, i) = c(j, i) * beta
            do k  = 1, nk
              c(j, i) = c(j, i) + (alpha * a(k, i) * b(j, k))
            end do
          end do
        end do
        end subroutine

      end program
