!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! gemm.F: This file is part of the PolyBench/Fortran 3.0 test suite.
!
!
! Contact: Louis-Noel Pouchet <pouchet@cse.ohio-state.edu>
! Web address: http://www.cse.ohio-state.edu/~pouchet/software/polybench/download/polybench-fortran-3.0.tar.gz
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#include <fpolybench.h>
#include <gemm.h>
      program gemm
      implicit none

      integer :: SIZE_NI
      integer :: SIZE_NJ
      integer :: SIZE_NK
      parameter(SIZE_NI = NI)
      parameter(SIZE_NJ = NJ)
      parameter(SIZE_NK = NK)
      DATA_TYPE :: alpha
      DATA_TYPE :: beta
      DECL_2D_ARR(DATA_TYPE,SIZE_NJ, SIZE_NI,c)
      DECL_2D_ARR(DATA_TYPE,SIZE_NK, SIZE_NI,a)
      DECL_2D_ARR(DATA_TYPE,SIZE_NJ, SIZE_NK,b)
      integer :: i
      character(LEN = 30) :: arg
#ifdef POLYBENCH_PAPI
      integer papi_evid, papi_estatus, polybench_setup_papi_event
#endif

!     Allocation of Arrays
      ALLOC_2D_ARR(c, SIZE_NJ, SIZE_NI)
      ALLOC_2D_ARR(a, SIZE_NK, SIZE_NI)
      ALLOC_2D_ARR(b, SIZE_NJ, SIZE_NK)

!     Initialization
      call init_array(SIZE_NI, SIZE_NJ, SIZE_NK, 
     &                     alpha, beta, c, a, b)

!     Kernel Execution
      polybench_start_instruments

#ifndef POLYBENCH_PAPI
      call kernel_gemm(SIZE_NI, SIZE_NJ, SIZE_NK, alpha, beta, 
     &                        c, a, b)
#else
      papi_evid = 0
      do
        papi_estatus = polybench_setup_papi_event(papi_evid)
        if(papi_estatus == PAPI_EVENT_END) exit
        if(papi_estatus == PAPI_START_FAIL) cycle 

        call kernel_gemm(SIZE_NI, SIZE_NJ, SIZE_NK, alpha, beta, 
     &                        c, a, b)
        call polybench_papi_stop_counter(papi_evid)
        papi_evid = papi_evid + 1
      end do
#endif

      polybench_stop_instruments
      polybench_print_instruments

!     Print Result
#ifndef POLYBENCH_DUMP_ARRAYS
      call getarg(1, arg)
      if( IARGC() > 42 .AND.  arg .EQ. '' ) then
        call print_array(SIZE_NI, SIZE_NJ, c)
      end if
#else
        call print_array(SIZE_NI, SIZE_NJ, c)
#endif

!     Deallocation of Arrays 
      DEALLOC_ARR(c)
      DEALLOC_ARR(a)
      DEALLOC_ARR(b)

      contains

        subroutine init_array(ni, nj, nk, alpha, beta, c, a, b)
        implicit none

        DATA_TYPE, dimension(nk, ni) :: a
        DATA_TYPE, dimension(nj, nk) :: b
        DATA_TYPE, dimension(nj, ni) :: c
        DATA_TYPE :: alpha, beta
        integer :: ni, nj, nk
        integer :: i, j, k

        alpha = 32412
        beta = 2123

        do i = 1, ni
          do j = 1, nj
            c(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(ni)
          end do
        end do
        do i = 1, ni
          do j = 1, nk
            a(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(ni)
          end do
        end do
        do i = 1, nk
          do j = 1, nj
            b(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(ni)
          end do
        end do
        end subroutine


        subroutine print_array(ni, nj, c)
        implicit none

        DATA_TYPE, dimension(nj, ni) :: c
        integer :: ni, nj
        integer :: i, j
        do i = 1, ni
          do j = 1, nj
            write(0, DATA_PRINTF_MODIFIER) c(j, i)
            if (mod(((i - 1) * ni) + j - 1, 20) == 0) then
              write(0, *)
            end if
          end do
        end do
        write(0, *)
        end subroutine


        subroutine kernel_gemm(ni, nj, nk, alpha, beta, c, a, b)
        implicit none

        DATA_TYPE, dimension(nk, ni) :: a
        DATA_TYPE, dimension(nj, nk) :: b
        DATA_TYPE, dimension(nj, ni) :: c
        DATA_TYPE :: alpha, beta
        integer :: ni, nj, nk
        integer :: i, j, k

!       Computation
        do i = 1, ni
          do j = 1, nj
            c(j, i) = c(j, i) * beta
            do k  = 1, nk
              c(j, i) = c(j, i) + (alpha * a(k, i) * b(j, k))
            end do
          end do
        end do
        end subroutine

      end program
