!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! gemver.F: This file is part of the PolyBench/Fortran 3.0 test suite.
!
!
! Contact: Louis-Noel Pouchet <pouchet@cse.ohio-state.edu>
! Web address: http://www.cse.ohio-state.edu/~pouchet/software/polybench/download/polybench-fortran-3.0.tar.gz
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include <fpolybench.h>
#include <gemver.h>

      program gemver
      implicit none

      integer :: SIZE_N
      parameter(SIZE_N = N)
      DATA_TYPE :: alpha
      DATA_TYPE :: beta
      DECL_2D_ARR(DATA_TYPE,SIZE_N, SIZE_N,a)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,u1)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,u2)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,v1)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,v2)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,w)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,x)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,y)
      DECL_1D_ARR(DATA_TYPE,SIZE_N,z)
      integer :: i
      character(LEN = 30) :: arg
#ifdef POLYBENCH_PAPI
      integer papi_evid, papi_estatus, polybench_setup_papi_event
#endif

!     Allocation of Arrays
      ALLOC_2D_ARR(a, SIZE_N, SIZE_N)
      ALLOC_1D_ARR(u1, SIZE_N)
      ALLOC_1D_ARR(u2, SIZE_N)
      ALLOC_1D_ARR(v1, SIZE_N)
      ALLOC_1D_ARR(v2, SIZE_N)
      ALLOC_1D_ARR(w, SIZE_N)
      ALLOC_1D_ARR(x, SIZE_N)
      ALLOC_1D_ARR(y, SIZE_N)
      ALLOC_1D_ARR(z, SIZE_N)

!     Initialization
      call init_array(SIZE_N, 
     &     alpha, beta, a, u1, u2, v1, v2, w, x, y, z)

!     Kernel Execution
      polybench_start_instruments

#ifndef POLYBENCH_PAPI
      call kernel_gemver(SIZE_N, alpha, beta,
     &                        a, u1, v1, u2, v2,
     &                        w, x, y, z)
#else
      papi_evid = 0
      do
        papi_estatus = polybench_setup_papi_event(papi_evid)
        if(papi_estatus == PAPI_EVENT_END) exit
        if(papi_estatus == PAPI_START_FAIL) cycle 

        call kernel_gemver(SIZE_N, alpha, beta,
     &                        a, u1, v1, u2, v2,
     &                        w, x, y, z)
        call polybench_papi_stop_counter(papi_evid)
        papi_evid = papi_evid + 1
      end do
#endif

      polybench_stop_instruments
      polybench_print_instruments

!     Print Result
#ifndef POLYBENCH_DUMP_ARRAYS
      call getarg(1, arg)
      if( IARGC() > 42 .AND.  arg .EQ. '' ) then
         call print_array(SIZE_N, w)
      end if
#else
         call print_array(SIZE_N, w)
#endif

!     Deallocation of Arrays 
      DEALLOC_ARR(a)
      DEALLOC_ARR(u1)
      DEALLOC_ARR(u2)
      DEALLOC_ARR(v1)
      DEALLOC_ARR(v2)
      DEALLOC_ARR(w)
      DEALLOC_ARR(x)
      DEALLOC_ARR(y)
      DEALLOC_ARR(z)

      contains

        subroutine init_array(n, alpha, beta, 
     &           a, u1, u2, v1, v2, w, x, y, z)
        implicit none

        DATA_TYPE, dimension(n, n) :: a
        DATA_TYPE, dimension(n) :: u1
        DATA_TYPE, dimension(n) :: u2
        DATA_TYPE, dimension(n) :: v1
        DATA_TYPE, dimension(n) :: v2
        DATA_TYPE, dimension(n) :: w
        DATA_TYPE, dimension(n) :: x
        DATA_TYPE, dimension(n) :: y
        DATA_TYPE, dimension(n) :: z
        DATA_TYPE :: alpha, beta
        integer :: n
        integer :: i, j
        alpha = 43532.0D0
        beta = 12313.0D0

        do i = 1, n
          u1(i) = DBLE(i - 1)
          u2(i) = DBLE(i / n) / 2.0D0
          v1(i) = DBLE(i / n) / 4.0D0
          v2(i) = DBLE(i / n) / 6.0D0
          y(i) = DBLE(i / n) / 8.0D0
          z(i) = DBLE(i / n) / 9.0D0
          x(i) = 0.0D0
          w(i) = 0.0D0
          do j = 1, n
            a(j, i) = ((DBLE(i - 1) * DBLE(j - 1))) / DBLE(n)
          end do
        end do
        end subroutine


        subroutine print_array(n, w)
        implicit none

        DATA_TYPE, dimension(n) :: w
        integer :: n
        integer :: i, j
        do i = 1, n
          write(0, DATA_PRINTF_MODIFIER) w(i)
          if (mod(i - 1, 20) == 0) then
            write(0, *)
          end if
        end do
        write(0, *)
        end subroutine
       

        subroutine kernel_gemver(n, alpha, beta, 
     &                                a, u1, v1, u2, v2,
     &                                w, x, y, z)
        implicit none

        DATA_TYPE, dimension(n, n) :: a
        DATA_TYPE, dimension(n) :: u1
        DATA_TYPE, dimension(n) :: u2
        DATA_TYPE, dimension(n) :: v1
        DATA_TYPE, dimension(n) :: v2
        DATA_TYPE, dimension(n) :: w
        DATA_TYPE, dimension(n) :: x
        DATA_TYPE, dimension(n) :: y
        DATA_TYPE, dimension(n) :: z
        DATA_TYPE :: alpha, beta
        integer :: n
        integer :: i, j

!       Computation
        do i = 1, n
          do j = 1, n
            a(j, i) = a(j, i) + (u1(i) * v1(j)) + (u2(i) * v2(j))
          end do
        end do
        do i = 1, n
          do j = 1, n
            x(i) = x(i) + (beta * a(i, j) * y(j))
          end do
        end do
        do i = 1, n
          x(i) = x(i) + z(i)
        end do
        do i = 1, n
          do j = 1, n
            w(i) = w(i) + (alpha * a(j, i) * x(j))
          end do
        end do
        end subroutine

      end program

