/**
 * polybench.h: This file is part of the PolyBench-Fortran 3.0 test suite.
 *
 *
 * Contact: Louis-Noel Pouchet <pouchet@cse.ohio-state.edu>
 */
/*
 * Polybench header for instrumentation.
 *
 * Programs must be compiled with `-I utilities utilities/polybench.c'
 *
 * Optionally, one can define:
 *
 * -DPOLYBENCH_TIME, to report the execution time,
 *   OR (exclusive):
 * -DPOLYBENCH_PAPI, to use PAPI H/W counters (defined in polybench.c)
 *
 */
#ifndef FPOLYBENCH_H
# define FPOLYBENCH_H

/* Array padding. By default, none is used. */
# ifndef POLYBENCH_PADDING_FACTOR
/* default: */
#  define POLYBENCH_PADDING_FACTOR 0
# endif

/* Initialize instruments macros to nothing */
#define polybench_start_instruments
#define polybench_stop_instruments
#define polybench_print_instruments

/* PAPI support */
#ifdef POLYBENCH_PAPI
#undef polybench_start_instruments
#undef polybench_stop_instruments
#undef polybench_print_instruments

#define polybench_start_instruments call polybench_prepare_instruments(); call polybench_papi_init()
#define polybench_stop_instruments call polybench_papi_close()
#define polybench_print_instruments call polybench_papi_print()

#define PAPI_EVENT_END 0
#define PAPI_START_FAIL 1
#define PAPI_START_SUCCESS 2
#endif

/* Timing support. */
#ifdef POLYBENCH_TIME
#undef polybench_start_instruments
#undef polybench_stop_instruments
#undef polybench_print_instruments
# define polybench_start_instruments call polybench_timer_start();
# define polybench_stop_instruments call polybench_timer_stop();
# define polybench_print_instruments call polybench_timer_print();
#endif


/* Macros for array declaration */
#ifndef POLYBENCH_STACK_ARRAYS
/* Heap */
#define DECL_1D_ARR(TYPE, SIZE, NAME) TYPE, dimension(:), allocatable :: NAME
#define DECL_2D_ARR(TYPE, SIZE1, SIZE2, NAME) TYPE, dimension(:,:), allocatable :: NAME
#define DECL_3D_ARR(TYPE, SIZE1, SIZE2, SIZE3, NAME) TYPE, dimension(:,:,:), allocatable :: NAME

#define ALLOC_1D_ARR(NAME, SIZE) allocate(NAME(SIZE+POLYBENCH_PADDING_FACTOR), STAT=I); call check_err(I) 
#define ALLOC_2D_ARR(NAME, SIZE1, SIZE2) allocate(NAME(SIZE1+POLYBENCH_PADDING_FACTOR,SIZE2+POLYBENCH_PADDING_FACTOR), STAT=I); call check_err(I) 
#define ALLOC_3D_ARR(NAME, SIZE1, SIZE2, SIZE3) allocate(NAME(SIZE1+POLYBENCH_PADDING_FACTOR,SIZE2+POLYBENCH_PADDING_FACTOR,SIZE3+POLYBENCH_PADDING_FACTOR), STAT=I); call check_err(I) 

#define DEALLOC_ARR(NAME) deallocate(NAME)

#else
/* Stack */
#define DECL_1D_ARR(TYPE, SIZE, NAME) TYPE, dimension(SIZE+POLYBENCH_PADDING_FACTOR) :: NAME
#define DECL_2D_ARR(TYPE, SIZE1, SIZE2, NAME) TYPE, dimension(SIZE1+POLYBENCH_PADDING_FACTOR, SIZE2+POLYBENCH_PADDING_FACTOR) :: NAME
#define DECL_3D_ARR(TYPE, SIZE1, SIZE2, SIZE3, NAME) TYPE, dimension(SIZE1+POLYBENCH_PADDING_FACTOR, SIZE2+POLYBENCH_PADDING_FACTOR, SIZE3+POLYBENCH_PADDING_FACTOR) :: NAME

#define ALLOC_1D_ARR(NAME, SIZE)
#define ALLOC_2D_ARR(NAME, SIZE1, SIZE2)
#define ALLOC_3D_ARR(NAME, SIZE1, SIZE2, SIZE3)

#define DEALLOC_ARR(NAME)
#endif

#endif
